# Copyright (C) 2026 GoodData Corporation

"""Output policy for MCP tool responses.

These caps keep responses small and deterministic while full logs are stored on disk.
"""

MAX_RESULT_BYTES = 12_000
MAX_RESULT_CHARS = 12_000
MAX_MCP_RESPONSE_BYTES = 64_000

MAX_ERROR_LINES = 20
MAX_SUMMARY_LINES = 12
MAX_ERROR_LINE_CHARS = 500
MAX_LOG_LINE_CHARS = 2_000
MAX_RESULT_LINE_CHARS = 500
MAX_PROGRESS_MESSAGE_CHARS = 400
MAX_STREAM_LINE_BYTES = 20_000

LOG_TAIL_MAX_BYTES = 20_000
LOG_TAIL_MAX_LINES = 400
